#include <u/libu.h>
#include "list.h"

typedef struct list_item_s
{
    TAILQ_ENTRY(list_item_s) np;
    void *ptr;
} list_item_t;

struct list_s
{
    TAILQ_HEAD(list_head_s, list_item_s) head;
    size_t count;
};

int list_create(list_t **plist)
{
    list_t *list = NULL;

    list = u_zalloc(sizeof(list_t));
    dbg_err_if(list == NULL); 

    TAILQ_INIT(&list->head);

    *plist = list;

    return 0;
err:
    if(list)
        u_free(list);
    return ~0;
}

/* note the list doesn't own the pointers in it, the client must free them */
int list_free(list_t *list)
{
    list_item_t *item = NULL;

    dbg_err_if(list == NULL);

    while((item = TAILQ_FIRST(&list->head)) != NULL)
        TAILQ_REMOVE(&list->head, item, np);

    u_free(list);

    return 0;
err:
    return ~0;
}

int list_add(list_t *list, void *ptr)
{
    list_item_t *item = NULL;

    item = (list_item_t*)u_zalloc(sizeof(list_item_t));
    dbg_err_if(item == NULL);

    item->ptr = ptr;
        
    TAILQ_INSERT_TAIL(&list->head, item, np);
    list->count++;

    return 0;
err:
    if(item)
        u_free(item);
    return ~0;
}

int list_del(list_t *list, void *ptr)
{
    list_item_t *item = NULL;

    TAILQ_FOREACH(item, &list->head, np)
    {
        if(item->ptr == ptr)
        {
            TAILQ_REMOVE(&list->head, item, np);
            list->count--;
            return 0; /* removed */
        }
    }

    return ~0; /* not found */
}

size_t list_count(list_t *list)
{
    return list->count;
}

void* list_get_n(list_t *list, size_t n)
{
    list_item_t *item = NULL;

    TAILQ_FOREACH(item, &list->head, np)
    {
        if(n-- == 0)
            return item->ptr;
    }

    return NULL;
}

