#include <klone/klone.h>
#include <klapp/msgchan.h>
#include <klapp/myservice.h>

static void cpustat_periodic(msgchan_t *mc, void *arg, struct timeval *tv)
{
    /* call me again in 0.5 seconds */
    tv->tv_sec = 0;
    tv->tv_usec = 500000; 

    /* get cpustat and return a JSON object */
    msgchan_printf(mc, "{\"id\":\"cpustat\",\"data\":%d}", rand() % 100);     

    return;
}

static void ethstat_periodic(msgchan_t *mc, void *arg, struct timeval *tv)
{
    /* call me again in 0.4 seconds */
    tv->tv_sec = 0;
    tv->tv_usec = 400000; 

    /* get ethernet stat and return a JSON object */
    msgchan_printf(mc, "{\"id\":\"ethstat\",\"data\":%d}", rand() % 100);     

    return;
}

static void cpustat(msgchan_t *mc, void *arg, const char *id, const char *data)
{
    info("%s msg received, data: [%s]", __FUNCTION__, data);

    /* register the cpustat_periodic to be called periodically */
    warn_if(msgchan_register_periodic(mc, cpustat_periodic, NULL));

    return;
}

static void ethstat(msgchan_t *mc, void *arg, const char *id, const char *data)
{
    info("%s msg received, data: [%s]", __FUNCTION__, data);

    /* register the ethstat_periodic to be called periodically */
    warn_if(msgchan_register_periodic(mc, ethstat_periodic, NULL));

    return;
}

/* dispatch messages and run periodic functions until the user leaves the web 
   page */
int myservice_dispatch(io_t *in, io_t *out, kaddr_t *server, kaddr_t *client)
{
    msgchan_t *mc = NULL;

    u_unused_args(server, client);

    /* create the message channel */
    crit_err_if(msgchan_create(in, out, &mc));

    /* register the callback functions for each supported message */
    crit_err_if(msgchan_register(mc, "cpustat", cpustat, NULL));
    crit_err_if(msgchan_register(mc, "ethstat", ethstat, NULL));
    
    /* dispatch messages until the client leaves the web page */
    crit_err_if(msgchan_dispatch(mc));

    msgchan_free(mc); mc = NULL;

    return 0;
err:
    if(mc)
        msgchan_free(mc);
    return ~0;
}
